"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var jimpex_exports = {};
__export(jimpex_exports, {
  Jimpex: () => Jimpex,
  jimpex: () => jimpex
});
module.exports = __toCommonJS(jimpex_exports);
var path = __toESM(require("path"));
var import_promises = __toESM(require("fs/promises"));
var import_https = require("https");
var import_jimple = require("@homer0/jimple");
var import_deep_assign = require("@homer0/deep-assign");
var import_simple_logger = require("@homer0/simple-logger");
var import_env_utils = require("@homer0/env-utils");
var import_package_info = require("@homer0/package-info");
var import_path_utils = require("@homer0/path-utils");
var import_root_file = require("@homer0/root-file");
var import_events_hub = require("@homer0/events-hub");
var import_simple_config = require("@homer0/simple-config");
var import_compression = __toESM(require("compression"));
var import_body_parser = __toESM(require("body-parser"));
var import_multer = __toESM(require("multer"));
var import_spdy = require("spdy");
var import_express = __toESM(require("express"));
var import_services = require("../services");
var import_utils = require("../utils");
class Jimpex extends import_jimple.Jimple {
  constructor(options = {}, config = {}) {
    super();
    __publicField(this, "_options");
    __publicField(this, "_express");
    __publicField(this, "_configReady", false);
    __publicField(this, "_server");
    __publicField(this, "_instance");
    __publicField(this, "_mountQueue", []);
    __publicField(this, "_controlledRoutes", []);
    this._options = (0, import_deep_assign.deepAssignWithOverwrite)(
      {
        filesizeLimit: "15MB",
        boot: true,
        path: {
          appPath: "",
          useParentPath: true
        },
        config: {
          default: options?.config?.default || config,
          name: "app",
          path: "config/",
          hasFolder: false,
          loadFromEnvironment: true,
          environmentVariable: "CONFIG",
          defaultConfigFilename: "[app-name].config.js",
          filenameFormat: "[app-name].[config-name].config.js"
        },
        statics: {
          enabled: true,
          onHome: false,
          route: "statics"
        },
        express: {
          trustProxy: true,
          disableXPoweredBy: true,
          compression: true,
          bodyParser: true,
          multer: true
        },
        services: {
          common: true,
          http: true,
          utils: true
        },
        healthCheck: () => Promise.resolve(true)
      },
      options,
      this._initOptions()
    );
    this._express = (0, import_express.default)();
    this._setupCoreServices();
    this._setupExpress();
    this._configurePath();
    this._init();
    if (this._options.boot) {
      this.boot();
    }
  }
  boot() {
  }
  disableTLSValidation() {
    process.env["NODE_TLS_REJECT_UNAUTHORIZED"] = "0";
    this.logger.warn("TLS validation has been disabled");
  }
  async start(onStart) {
    await this._setupConfig();
    const config = this.getConfig();
    const port = config.get("port");
    if (!port) {
      throw new Error("No port configured");
    }
    this._emitEvent("beforeStart", { app: this });
    this._server = await this._createServer();
    this._instance = this._server.listen(port, () => {
      this._emitEvent("start", { app: this });
      this._mountResources();
      this.logger.success(`Starting on port ${port}`);
      this._emitEvent("afterStart", { app: this });
      if (onStart) {
        onStart(config);
      }
      this._emitEvent("afterStartCallback", { app: this });
    });
    return this._instance;
  }
  async listen(port, onStart) {
    if (port) {
      await this._setupConfig();
      const config = this.getConfig();
      config.set("port", port);
    }
    return this.start(onStart);
  }
  stop() {
    if (!this._instance)
      return;
    this._emitEvent("beforeStop", { app: this });
    this._instance.close();
    this._instance = void 0;
    this._emitEvent("afterStop", { app: this });
  }
  mount(route, controller) {
    let useController;
    if ("register" in controller && typeof controller.register === "function" && controller.provider === true) {
      useController = controller.register(this, route);
    } else if ("connect" in controller && typeof controller.connect === "function" && ("middleware" in controller && controller.middleware === true || "controller" in controller && controller.controller === true)) {
      useController = controller;
    } else {
      useController = {
        middleware: true,
        connect: () => controller
      };
    }
    this._mountQueue.push((server) => {
      const connected = useController.connect(this, route);
      if (!connected)
        return;
      const router = this._reduceWithEvent("controllerWillBeMounted", connected, {
        route,
        controller: useController,
        app: this
      });
      server.use(route, router);
      this._emitEvent("routeAdded", { route, app: this });
      this._controlledRoutes.push(route);
    });
  }
  use(middleware) {
    const useMiddleware = "register" in middleware && typeof middleware.register === "function" ? middleware.register(this) : middleware;
    this._mountQueue.push((server) => {
      if ("connect" in useMiddleware && typeof useMiddleware.connect === "function") {
        const handler = useMiddleware.connect(this);
        if (handler) {
          server.use(
            this._reduceWithEvent("middlewareWillBeUsed", handler, { app: this })
          );
        }
        return;
      }
      server.use(
        this._reduceWithEvent(
          "middlewareWillBeUsed",
          useMiddleware,
          { app: this }
        )
      );
    });
  }
  getConfig(setting, asArray = false) {
    const config = this.try("config");
    if (!config) {
      throw new Error("The config service is not available until the app starts");
    }
    if (typeof setting === "undefined") {
      return config;
    }
    return config.get(setting, asArray);
  }
  getRouter() {
    return this.get("router");
  }
  get logger() {
    return this.get("logger");
  }
  get express() {
    return this._express;
  }
  get instance() {
    return this._instance;
  }
  get options() {
    return (0, import_deep_assign.deepAssignWithOverwrite)({}, this._options);
  }
  get eventsHub() {
    return this.get("events");
  }
  get routes() {
    return this._controlledRoutes.slice();
  }
  on(eventName, listener) {
    return this.eventsHub.on(eventName, listener);
  }
  once(eventName, listener) {
    return this.eventsHub.once(eventName, listener);
  }
  isHealthy() {
    return this._options.healthCheck(this);
  }
  _init() {
  }
  _initOptions() {
    return {};
  }
  _setupCoreServices() {
    this.register(
      (0, import_simple_logger.appLoggerProvider)({
        serviceName: "logger"
      })
    );
    this.register(import_env_utils.envUtilsProvider);
    this.register(import_package_info.packageInfoProvider);
    this.register(import_path_utils.pathUtilsProvider);
    this.register(import_root_file.rootFileProvider);
    const { services: enabledServices } = this._options;
    if (enabledServices.common)
      this.register(import_services.commonServicesProvider);
    if (enabledServices.http)
      this.register(import_services.httpServicesProvider);
    if (enabledServices.utils)
      this.register(import_services.utilsServicesProvider);
    this.set("events", () => new import_events_hub.EventsHub());
    this.set("statuses", () => import_utils.statuses);
  }
  _setupExpress() {
    const { statics, filesizeLimit, express: expressOptions } = this._options;
    if (expressOptions.trustProxy) {
      this._express.enable("trust proxy");
    }
    if (expressOptions.disableXPoweredBy) {
      this._express.disable("x-powered-by");
    }
    if (expressOptions.compression) {
      this._express.use((0, import_compression.default)());
    }
    if (statics.enabled) {
      this._addStaticsFolder(statics.route, statics.folder, statics.onHome);
    }
    if (expressOptions.bodyParser) {
      this._express.use(
        import_body_parser.default.json({
          limit: filesizeLimit
        })
      );
      this._express.use(
        import_body_parser.default.urlencoded({
          extended: true,
          limit: filesizeLimit
        })
      );
    }
    if (expressOptions.multer) {
      this._express.use((0, import_multer.default)().any());
    }
    this.set(
      "router",
      this.factory(() => import_express.default.Router())
    );
  }
  _addStaticsFolder(route, folder = "", onHome = false) {
    const location = onHome ? "home" : "app";
    const staticRoute = route.replace(/^\/+/, "");
    const pathUtils = this.get("pathUtils");
    const staticFolder = pathUtils.joinFrom(location, folder || staticRoute);
    this.mount(`/${staticRoute}`, {
      connect: () => import_express.default.static(staticFolder),
      controller: true
    });
  }
  _configurePath() {
    const pathUtils = this.get("pathUtils");
    const {
      path: { appPath, useParentPath }
    } = this._options;
    if (appPath) {
      pathUtils.addLocation("app", appPath);
      return;
    }
    let foundPath = false;
    if (useParentPath) {
      const stack = new Error().stack;
      const stackList = stack.split("\n");
      stackList.shift();
      const parentFromStack = stackList.find((line) => !line.includes(__filename));
      if (parentFromStack) {
        const parentFile = parentFromStack.replace(/^.*?\s\(([^\s]+):\d+:\d+\)/, "$1");
        if (parentFile !== parentFromStack) {
          foundPath = true;
          pathUtils.addLocation("app", path.dirname(parentFile));
        }
      }
    }
    if (!foundPath) {
      throw new Error(
        "The app location cannot be determined. Please specify the appPath option."
      );
    }
  }
  async _setupConfig() {
    if (this._configReady)
      return;
    this._configReady = true;
    const { config: options } = this._options;
    let configsPath = options.path.replace(/\/$/, "");
    if (options.hasFolder) {
      configsPath = `${configsPath}${path.sep}${options.name}${path.sep}`;
    }
    const filenameFormat = options.filenameFormat.replace(/\[app-name\]/gi, options.name).replace(/\[config-name\]/gi, "[name]");
    const defaultConfigFilename = options.defaultConfigFilename.replace(
      /\[app-name\]/gi,
      options.name
    );
    this.register(
      (0, import_simple_config.simpleConfigProvider)({
        name: options.name,
        defaultConfig: options.default,
        defaultConfigFilename,
        envVarName: options.environmentVariable,
        path: configsPath,
        filenameFormat
      })
    );
    const config = this.getConfig();
    await config.loadFromFile("", true, false);
    if (options.loadFromEnvironment) {
      await config.loadFromEnv();
    }
  }
  _mountResources() {
    this._mountQueue.forEach((mount) => mount(this._express));
    this._mountQueue.length = 0;
  }
  _emitEvent(name, payload) {
    this.eventsHub.emit(name, payload);
  }
  _reduceWithEvent(name, target, payload) {
    return this.eventsHub.reduceSync(name, target, payload);
  }
  async _loadCredentials(credentialsInfo, onHome = true) {
    const location = onHome ? "home" : "app";
    const pathUtils = this.get("pathUtils");
    const keys = ["ca", "cert", "key"];
    const info = await Promise.all(
      keys.map(async (key) => {
        const filepath = credentialsInfo[key];
        if (!filepath)
          return void 0;
        const file = await import_promises.default.readFile(pathUtils.joinFrom(location, filepath), "utf8");
        return {
          key,
          file
        };
      })
    );
    return info.reduce((acc, item) => {
      if (item) {
        acc[item.key] = item.file;
      }
      return acc;
    }, {});
  }
  async _createServer() {
    const [http2Config = {}, httpsConfig = {}] = this.getConfig(["http2", "https"], true);
    if (!http2Config.enabled && !httpsConfig.enabled) {
      return this._express;
    }
    if (http2Config.enabled && !httpsConfig.enabled) {
      throw new Error("HTTP2 requires for HTTPS to be enabled");
    }
    if (!httpsConfig.credentials) {
      throw new Error("The `credentials` object on the HTTPS settings is missing");
    }
    const credentials = await this._loadCredentials(
      httpsConfig.credentials,
      httpsConfig.credentials.onHome
    );
    if (!Object.keys(credentials).length) {
      throw new Error("No credentials were found for HTTPS");
    }
    if (http2Config.enabled) {
      const serverOptions = {
        ...credentials,
        spdy: http2Config.spdy
      };
      return (0, import_spdy.createServer)(serverOptions, this._express);
    }
    return (0, import_https.createServer)(credentials, this._express);
  }
}
const jimpex = (...args) => new Jimpex(...args);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Jimpex,
  jimpex
});
//# sourceMappingURL=jimpex.js.map