"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var gateway_exports = {};
__export(gateway_exports, {
  GatewayController: () => GatewayController,
  gatewayController: () => gatewayController
});
module.exports = __toCommonJS(gateway_exports);
var import_deep_assign = require("@homer0/deep-assign");
var import_object_utils = require("@homer0/object-utils");
var import_utils = require("../../utils");
class GatewayController {
  constructor({
    inject,
    route,
    gatewayConfig,
    ...options
  }) {
    __publicField(this, "http");
    __publicField(this, "_getHelperService");
    __publicField(this, "_gatewayConfig");
    __publicField(this, "_route");
    __publicField(this, "_routeExpression");
    __publicField(this, "_options");
    __publicField(this, "_endpoints");
    __publicField(this, "_apiConfigUrl");
    __publicField(this, "_apiConfigEndpoints");
    __publicField(this, "_routes");
    this.http = inject.http;
    this._getHelperService = inject.getHelperService || (() => void 0);
    this._route = (0, import_utils.removeSlashes)(route);
    this._options = this._formatOptions(
      (0, import_deep_assign.deepAssignWithOverwrite)(
        {
          root: "",
          apiConfigSetting: "api",
          headers: {
            useXForwardedFor: true,
            copyCustomHeaders: true,
            copy: ["authorization", "content-type", "referer", "user-agent"],
            remove: ["server", "x-powered-by", "content-encoding"]
          }
        },
        options
      )
    );
    this._gatewayConfig = {
      ...gatewayConfig,
      url: (0, import_utils.removeSlashes)(gatewayConfig.url, false, true)
    };
    this._routeExpression = (0, import_utils.createRouteExpression)(
      this._options.root ? `${this._route}/${this._options.root}` : this._route,
      true,
      true
    );
    this._endpoints = this._formatEndpoints();
    this._routes = this._createRoutes();
    const { url, endpoints } = this._createAPIConfig();
    this._apiConfigUrl = url;
    this._apiConfigEndpoints = endpoints;
  }
  getAPIConfig({
    setting,
    placeholders = {}
  } = {}) {
    const useSetting = setting || this._options.apiConfigSetting;
    let url;
    const placeholdersEntries = Object.entries(placeholders);
    if (placeholdersEntries.length) {
      url = placeholdersEntries.reduce(
        (acc, [key, value]) => acc.replace(`:${key}`, value),
        this._apiConfigUrl
      );
    } else {
      url = this._apiConfigUrl;
    }
    return {
      url,
      endpoints: {
        [useSetting]: this._apiConfigEndpoints
      }
    };
  }
  addRoutes(router, middlewares = []) {
    this._routes.forEach((route) => {
      route.methods.forEach((info) => {
        this._addRoute({
          router,
          method: info.method,
          route: route.route,
          gatewayMiddleware: this._createGatewayMiddleware(info.endpoint),
          middlewares
        });
      });
    });
    return router;
  }
  get options() {
    return { ...this._options };
  }
  get gatewayConfig() {
    return { ...this._gatewayConfig };
  }
  _createGatewayMiddleware(endpoint) {
    return async (req, res, next) => {
      const {
        _options: { headers: headersOptions },
        _gatewayConfig: { url: gatewayUrl },
        _routeExpression: routeExpression
      } = this;
      const reqPath = req.originalUrl.replace(routeExpression, "");
      let headers = {};
      headersOptions.copy.forEach((name) => {
        if (req.headers[name]) {
          headers[name] = req.headers[name];
        }
      });
      if (headersOptions.copyCustomHeaders) {
        headers = (0, import_deep_assign.deepAssignWithOverwrite)(
          headers,
          this.http.getCustomHeadersFromRequest(req)
        );
      }
      if (headersOptions.useXForwardedFor) {
        const ip = this.http.getIPFromRequest(req);
        if (ip) {
          headers["x-forwarded-for"] = ip;
        }
      }
      const method = req.method.toUpperCase();
      let body;
      if (method !== "GET" && typeof req.body === "object") {
        body = JSON.stringify(req.body);
        if (!headers["content-type"]) {
          headers["content-type"] = "application/json";
        }
      }
      const helper = this._getHelperService() || {};
      const helperBasePayload = {
        endpoint,
        req,
        res,
        next,
        helper
      };
      try {
        const request = await this._reduceEndpointRequest({
          endpointReq: {
            url: `${gatewayUrl}/${reqPath}`,
            options: {
              method,
              headers,
              body
            }
          },
          ...helperBasePayload
        });
        const responseRaw = await this.http.fetch(request.url, request.options);
        const response = await this._reduceEndpointResponse({
          endpointRes: responseRaw,
          ...helperBasePayload
        });
        const shouldStream = await this._shouldStreamEndpointResponse({
          endpointRes: responseRaw,
          ...helperBasePayload
        });
        if (shouldStream) {
          res.status(response.status);
          response.headers.forEach((value, name) => {
            if (!headersOptions.remove.includes(name)) {
              res.setHeader(name, value);
            }
          });
          response.body.pipe(res).on("error", (error) => {
            next(error);
          });
        } else {
          await this._handleEndpointResponse({
            endpointRes: response,
            ...helperBasePayload
          });
        }
      } catch (error) {
        this._handleEndpointError({
          error,
          ...helperBasePayload
        });
      }
    };
  }
  _addRoute({
    router,
    method,
    route,
    gatewayMiddleware,
    middlewares
  }) {
    router[method](route, [...middlewares, gatewayMiddleware]);
  }
  _formatEndpoints() {
    return (0, import_object_utils.flat)({
      target: this._gatewayConfig.gateway,
      shouldFlatten: (_, value) => {
        const useValue = value;
        return typeof useValue.path === "undefined";
      }
    });
  }
  _createRoutes() {
    const routes = {};
    Object.keys(this._endpoints).forEach((name) => {
      const endpoint = this._endpoints[name];
      let endpointPath;
      let endpointMethod;
      if (typeof endpoint === "string") {
        endpointPath = endpoint;
        endpointMethod = "all";
      } else {
        endpointPath = endpoint.path;
        endpointMethod = endpoint.method ? this._validateHTTPMethod(endpoint.method) : "all";
      }
      endpointPath = (0, import_utils.removeSlashes)(endpointPath);
      if (!routes[endpointPath]) {
        routes[endpointPath] = {
          path: endpointPath,
          methods: {}
        };
      }
      if (routes[endpointPath].methods[endpointMethod]) {
        const repeatedEndpoint = routes[endpointPath].methods[endpointMethod];
        throw new Error(
          `You can't have two gateway endpoints to the same path and with the same HTTP method: '${repeatedEndpoint}' and '${name}'`
        );
      }
      routes[endpointPath].methods[endpointMethod] = name;
    });
    const routePrefixes = this._options.root ? `/${this._options.root}/` : "/";
    return Object.keys(routes).map((endpointPath) => {
      const info = routes[endpointPath];
      return {
        path: info.path,
        route: `${routePrefixes}${info.path}`,
        methods: Object.keys(info.methods).map((methodNameRaw) => {
          const methodName = methodNameRaw;
          const endpointName = info.methods[methodName];
          const endpointDefinition = this._endpoints[endpointName];
          return {
            method: methodName,
            endpoint: {
              name: endpointName,
              definition: endpointDefinition
            }
          };
        })
      };
    });
  }
  _reduceEndpointRequest({
    helper,
    ...options
  }) {
    if (helper.reduceEndpointRequest) {
      return helper.reduceEndpointRequest(options);
    }
    return Promise.resolve(options.endpointReq);
  }
  _reduceEndpointResponse({
    helper,
    ...options
  }) {
    if (helper.reduceEndpointResponse) {
      return helper.reduceEndpointResponse(options);
    }
    return Promise.resolve(options.endpointRes);
  }
  _shouldStreamEndpointResponse({
    helper,
    ...options
  }) {
    if (helper.shouldStreamEndpointResponse) {
      return helper.shouldStreamEndpointResponse(options);
    }
    return Promise.resolve(true);
  }
  _handleEndpointResponse({
    helper,
    ...options
  }) {
    if (!helper.handleEndpointResponse) {
      throw new Error("You must implement handleEndpointResponse");
    }
    return helper.handleEndpointResponse(options);
  }
  _handleEndpointError({
    helper,
    ...options
  }) {
    if (helper.handleEndpointError) {
      return helper.handleEndpointError(options);
    }
    return options.next(options.error);
  }
  _formatOptions(options) {
    if (options.root) {
      const root = (0, import_utils.removeSlashes)(options.root).trim();
      return { ...options, root };
    }
    return options;
  }
  _validateHTTPMethod(method) {
    const newMethod = method.toLowerCase();
    return [
      "get",
      "head",
      "post",
      "patch",
      "put",
      "delete",
      "connect",
      "options",
      "trace"
    ].includes(newMethod) ? newMethod : "all";
  }
  _createAPIConfig() {
    let endpoints;
    const { root } = this._options;
    if (root) {
      endpoints = Object.keys(this._endpoints).reduce((acc, name) => {
        const endpoint = this._endpoints[name];
        let newEndpoint;
        if (typeof endpoint === "string") {
          newEndpoint = (0, import_utils.removeSlashes)(endpoint);
          newEndpoint = `${root}/${newEndpoint}`;
        } else {
          const endpointPath = (0, import_utils.removeSlashes)(endpoint.path);
          newEndpoint = {
            ...endpoint,
            path: `${root}/${endpointPath}`
          };
        }
        acc[name] = newEndpoint;
        return acc;
      }, {});
    } else {
      endpoints = this._endpoints;
    }
    return {
      url: `/${this._route}`,
      endpoints: (0, import_object_utils.unflat)({ target: endpoints })
    };
  }
}
const gatewayController = (0, import_utils.controllerProviderCreator)(
  (options = {}) => (app, route) => {
    const defaultServiceName = "apiGateway";
    let defaultHelperServiceName = "apiGatewayHelper";
    let defaultConfigSetting = "api";
    let { serviceName = defaultServiceName } = options;
    if (serviceName !== defaultServiceName) {
      defaultConfigSetting = serviceName;
      if (!serviceName.match(/gateway$/i)) {
        serviceName = `${serviceName}Gateway`;
      }
      defaultHelperServiceName = `${serviceName}Helper`;
    }
    app.set(serviceName, () => {
      const {
        helperServiceName = defaultHelperServiceName,
        gatewaySettingName = defaultConfigSetting,
        gatewayClass: GatewayClass = GatewayController
      } = options;
      const gtConfig = app.getConfig(gatewaySettingName);
      return new GatewayClass({
        ...options,
        apiConfigSetting: gatewaySettingName,
        gatewayConfig: gtConfig,
        route,
        inject: {
          http: app.get("http"),
          getHelperService: () => app.try(helperServiceName)
        }
      });
    });
    return (0, import_utils.controller)(() => {
      const ctrl = app.get(serviceName);
      let useMiddlewares;
      if (options.getMiddlewares) {
        useMiddlewares = options.getMiddlewares(app).map((middleware) => {
          if ("middleware" in middleware) {
            return middleware.connect(app);
          }
          return middleware;
        }).filter(import_utils.notUndefined);
      }
      return ctrl.addRoutes(app.get("router"), useMiddlewares);
    });
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  GatewayController,
  gatewayController
});
//# sourceMappingURL=gateway.js.map