import {
  __publicField
} from "../chunk-T2T6Q22Z.js";
import * as path from "path";
import fs from "fs/promises";
import { createServer as createHTTPSServer } from "https";
import { Jimple } from "@homer0/jimple";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { appLoggerProvider } from "@homer0/simple-logger";
import { envUtilsProvider } from "@homer0/env-utils";
import { packageInfoProvider } from "@homer0/package-info";
import { pathUtilsProvider } from "@homer0/path-utils";
import { rootFileProvider } from "@homer0/root-file";
import { EventsHub } from "@homer0/events-hub";
import { simpleConfigProvider } from "@homer0/simple-config";
import compression from "compression";
import bodyParser from "body-parser";
import multer from "multer";
import {
  createServer as createSpdyServer
} from "spdy";
import express from "express";
import {
  commonServicesProvider,
  httpServicesProvider,
  utilsServicesProvider
} from "../services";
import {
  statuses
} from "../utils";
class Jimpex extends Jimple {
  constructor(options = {}, config = {}) {
    super();
    __publicField(this, "_options");
    __publicField(this, "_express");
    __publicField(this, "_configReady", false);
    __publicField(this, "_server");
    __publicField(this, "_instance");
    __publicField(this, "_mountQueue", []);
    __publicField(this, "_controlledRoutes", []);
    this._options = deepAssignWithOverwrite(
      {
        filesizeLimit: "15MB",
        boot: true,
        path: {
          appPath: "",
          useParentPath: true
        },
        config: {
          default: options?.config?.default || config,
          name: "app",
          path: "config/",
          hasFolder: false,
          loadFromEnvironment: true,
          environmentVariable: "CONFIG",
          defaultConfigFilename: "[app-name].config.js",
          filenameFormat: "[app-name].[config-name].config.js"
        },
        statics: {
          enabled: true,
          onHome: false,
          route: "statics"
        },
        express: {
          trustProxy: true,
          disableXPoweredBy: true,
          compression: true,
          bodyParser: true,
          multer: true
        },
        services: {
          common: true,
          http: true,
          utils: true
        },
        healthCheck: () => Promise.resolve(true)
      },
      options,
      this._initOptions()
    );
    this._express = express();
    this._setupCoreServices();
    this._setupExpress();
    this._configurePath();
    this._init();
    if (this._options.boot) {
      this.boot();
    }
  }
  boot() {
  }
  disableTLSValidation() {
    process.env["NODE_TLS_REJECT_UNAUTHORIZED"] = "0";
    this.logger.warn("TLS validation has been disabled");
  }
  async start(onStart) {
    await this._setupConfig();
    const config = this.getConfig();
    const port = config.get("port");
    if (!port) {
      throw new Error("No port configured");
    }
    this._emitEvent("beforeStart", { app: this });
    this._server = await this._createServer();
    this._instance = this._server.listen(port, () => {
      this._emitEvent("start", { app: this });
      this._mountResources();
      this.logger.success(`Starting on port ${port}`);
      this._emitEvent("afterStart", { app: this });
      if (onStart) {
        onStart(config);
      }
      this._emitEvent("afterStartCallback", { app: this });
    });
    return this._instance;
  }
  async listen(port, onStart) {
    if (port) {
      await this._setupConfig();
      const config = this.getConfig();
      config.set("port", port);
    }
    return this.start(onStart);
  }
  stop() {
    if (!this._instance)
      return;
    this._emitEvent("beforeStop", { app: this });
    this._instance.close();
    this._instance = void 0;
    this._emitEvent("afterStop", { app: this });
  }
  mount(route, controller) {
    let useController;
    if ("register" in controller && typeof controller.register === "function" && controller.provider === true) {
      useController = controller.register(this, route);
    } else if ("connect" in controller && typeof controller.connect === "function" && ("middleware" in controller && controller.middleware === true || "controller" in controller && controller.controller === true)) {
      useController = controller;
    } else {
      useController = {
        middleware: true,
        connect: () => controller
      };
    }
    this._mountQueue.push((server) => {
      const connected = useController.connect(this, route);
      if (!connected)
        return;
      const router = this._reduceWithEvent("controllerWillBeMounted", connected, {
        route,
        controller: useController,
        app: this
      });
      server.use(route, router);
      this._emitEvent("routeAdded", { route, app: this });
      this._controlledRoutes.push(route);
    });
  }
  use(middleware) {
    const useMiddleware = "register" in middleware && typeof middleware.register === "function" ? middleware.register(this) : middleware;
    this._mountQueue.push((server) => {
      if ("connect" in useMiddleware && typeof useMiddleware.connect === "function") {
        const handler = useMiddleware.connect(this);
        if (handler) {
          server.use(
            this._reduceWithEvent("middlewareWillBeUsed", handler, { app: this })
          );
        }
        return;
      }
      server.use(
        this._reduceWithEvent(
          "middlewareWillBeUsed",
          useMiddleware,
          { app: this }
        )
      );
    });
  }
  getConfig(setting, asArray = false) {
    const config = this.try("config");
    if (!config) {
      throw new Error("The config service is not available until the app starts");
    }
    if (typeof setting === "undefined") {
      return config;
    }
    return config.get(setting, asArray);
  }
  getRouter() {
    return this.get("router");
  }
  get logger() {
    return this.get("logger");
  }
  get express() {
    return this._express;
  }
  get instance() {
    return this._instance;
  }
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
  get eventsHub() {
    return this.get("events");
  }
  get routes() {
    return this._controlledRoutes.slice();
  }
  on(eventName, listener) {
    return this.eventsHub.on(eventName, listener);
  }
  once(eventName, listener) {
    return this.eventsHub.once(eventName, listener);
  }
  isHealthy() {
    return this._options.healthCheck(this);
  }
  _init() {
  }
  _initOptions() {
    return {};
  }
  _setupCoreServices() {
    this.register(
      appLoggerProvider({
        serviceName: "logger"
      })
    );
    this.register(envUtilsProvider);
    this.register(packageInfoProvider);
    this.register(pathUtilsProvider);
    this.register(rootFileProvider);
    const { services: enabledServices } = this._options;
    if (enabledServices.common)
      this.register(commonServicesProvider);
    if (enabledServices.http)
      this.register(httpServicesProvider);
    if (enabledServices.utils)
      this.register(utilsServicesProvider);
    this.set("events", () => new EventsHub());
    this.set("statuses", () => statuses);
  }
  _setupExpress() {
    const { statics, filesizeLimit, express: expressOptions } = this._options;
    if (expressOptions.trustProxy) {
      this._express.enable("trust proxy");
    }
    if (expressOptions.disableXPoweredBy) {
      this._express.disable("x-powered-by");
    }
    if (expressOptions.compression) {
      this._express.use(compression());
    }
    if (statics.enabled) {
      this._addStaticsFolder(statics.route, statics.folder, statics.onHome);
    }
    if (expressOptions.bodyParser) {
      this._express.use(
        bodyParser.json({
          limit: filesizeLimit
        })
      );
      this._express.use(
        bodyParser.urlencoded({
          extended: true,
          limit: filesizeLimit
        })
      );
    }
    if (expressOptions.multer) {
      this._express.use(multer().any());
    }
    this.set(
      "router",
      this.factory(() => express.Router())
    );
  }
  _addStaticsFolder(route, folder = "", onHome = false) {
    const location = onHome ? "home" : "app";
    const staticRoute = route.replace(/^\/+/, "");
    const pathUtils = this.get("pathUtils");
    const staticFolder = pathUtils.joinFrom(location, folder || staticRoute);
    this.mount(`/${staticRoute}`, {
      connect: () => express.static(staticFolder),
      controller: true
    });
  }
  _configurePath() {
    const pathUtils = this.get("pathUtils");
    const {
      path: { appPath, useParentPath }
    } = this._options;
    if (appPath) {
      pathUtils.addLocation("app", appPath);
      return;
    }
    let foundPath = false;
    if (useParentPath) {
      const stack = new Error().stack;
      const stackList = stack.split("\n");
      stackList.shift();
      const parentFromStack = stackList.find((line) => !line.includes(__filename));
      if (parentFromStack) {
        const parentFile = parentFromStack.replace(/^.*?\s\(([^\s]+):\d+:\d+\)/, "$1");
        if (parentFile !== parentFromStack) {
          foundPath = true;
          pathUtils.addLocation("app", path.dirname(parentFile));
        }
      }
    }
    if (!foundPath) {
      throw new Error(
        "The app location cannot be determined. Please specify the appPath option."
      );
    }
  }
  async _setupConfig() {
    if (this._configReady)
      return;
    this._configReady = true;
    const { config: options } = this._options;
    let configsPath = options.path.replace(/\/$/, "");
    if (options.hasFolder) {
      configsPath = `${configsPath}${path.sep}${options.name}${path.sep}`;
    }
    const filenameFormat = options.filenameFormat.replace(/\[app-name\]/gi, options.name).replace(/\[config-name\]/gi, "[name]");
    const defaultConfigFilename = options.defaultConfigFilename.replace(
      /\[app-name\]/gi,
      options.name
    );
    this.register(
      simpleConfigProvider({
        name: options.name,
        defaultConfig: options.default,
        defaultConfigFilename,
        envVarName: options.environmentVariable,
        path: configsPath,
        filenameFormat
      })
    );
    const config = this.getConfig();
    await config.loadFromFile("", true, false);
    if (options.loadFromEnvironment) {
      await config.loadFromEnv();
    }
  }
  _mountResources() {
    this._mountQueue.forEach((mount) => mount(this._express));
    this._mountQueue.length = 0;
  }
  _emitEvent(name, payload) {
    this.eventsHub.emit(name, payload);
  }
  _reduceWithEvent(name, target, payload) {
    return this.eventsHub.reduceSync(name, target, payload);
  }
  async _loadCredentials(credentialsInfo, onHome = true) {
    const location = onHome ? "home" : "app";
    const pathUtils = this.get("pathUtils");
    const keys = ["ca", "cert", "key"];
    const info = await Promise.all(
      keys.map(async (key) => {
        const filepath = credentialsInfo[key];
        if (!filepath)
          return void 0;
        const file = await fs.readFile(pathUtils.joinFrom(location, filepath), "utf8");
        return {
          key,
          file
        };
      })
    );
    return info.reduce((acc, item) => {
      if (item) {
        acc[item.key] = item.file;
      }
      return acc;
    }, {});
  }
  async _createServer() {
    const [http2Config = {}, httpsConfig = {}] = this.getConfig(["http2", "https"], true);
    if (!http2Config.enabled && !httpsConfig.enabled) {
      return this._express;
    }
    if (http2Config.enabled && !httpsConfig.enabled) {
      throw new Error("HTTP2 requires for HTTPS to be enabled");
    }
    if (!httpsConfig.credentials) {
      throw new Error("The `credentials` object on the HTTPS settings is missing");
    }
    const credentials = await this._loadCredentials(
      httpsConfig.credentials,
      httpsConfig.credentials.onHome
    );
    if (!Object.keys(credentials).length) {
      throw new Error("No credentials were found for HTTPS");
    }
    if (http2Config.enabled) {
      const serverOptions = {
        ...credentials,
        spdy: http2Config.spdy
      };
      return createSpdyServer(serverOptions, this._express);
    }
    return createHTTPSServer(credentials, this._express);
  }
}
const jimpex = (...args) => new Jimpex(...args);
export {
  Jimpex,
  jimpex
};
//# sourceMappingURL=jimpex.js.map