import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { controller } from "../../utils";
class ConfigController {
  constructor({ inject }) {
    __publicField(this, "_responsesBuilder");
    __publicField(this, "_config");
    this._responsesBuilder = inject.responsesBuilder;
    this._config = inject.config;
  }
  showConfig() {
    return (_, res) => {
      this.respondWithConfig(res);
    };
  }
  switchConfig() {
    return async (req, res, next) => {
      const { name } = req.params;
      if (!name || !this._config.canSwitchConfigs()) {
        next();
        return;
      }
      try {
        await this._config.switch(name);
        this.respondWithConfig(res);
      } catch (error) {
        next(error);
      }
    };
  }
  respondWithConfig(res) {
    const name = this._config.get("name");
    const data = {
      name,
      ...this._config.getConfig()
    };
    this._responsesBuilder.json({
      res,
      data
    });
  }
}
const configController = controller((app) => {
  const config = app.getConfig();
  const router = app.getRouter();
  if (config.get("debug.configController") !== true) {
    return router;
  }
  const ctrl = new ConfigController({
    inject: {
      config,
      responsesBuilder: app.get("responsesBuilder")
    }
  });
  return router.get("/", ctrl.showConfig()).get("/switch/:name", ctrl.switchConfig());
});
export {
  ConfigController,
  configController
};
//# sourceMappingURL=config.js.map