import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { controller } from "../../utils";
class HealthController {
  constructor({ inject }) {
    __publicField(this, "_getHealthStatus");
    __publicField(this, "_responsesBuilder");
    __publicField(this, "_config");
    __publicField(this, "_statuses");
    this._getHealthStatus = inject.getHealthStatus;
    this._responsesBuilder = inject.responsesBuilder;
    this._config = inject.config;
    this._statuses = inject.statuses;
  }
  showHealth() {
    return async (_, res) => {
      const healthStatus = await this._getHealthStatus();
      let isHealthy;
      let extras;
      if (typeof healthStatus === "boolean") {
        isHealthy = healthStatus;
        extras = {};
      } else {
        if (typeof healthStatus.isHealthy === "boolean") {
          isHealthy = healthStatus.isHealthy;
        } else if (healthStatus.services) {
          isHealthy = Object.values(healthStatus.services).every(
            (value) => value === true
          );
        } else {
          isHealthy = true;
        }
        extras = {
          ...healthStatus.services
        };
      }
      const { name: config, version } = this._config.get(["name", "version"]);
      const status = isHealthy ? this._statuses("ok") : this._statuses("service unavailable");
      this._responsesBuilder.json({
        res,
        status,
        data: {
          isHealthy,
          status,
          config,
          version,
          ...extras
        }
      });
    };
  }
}
const healthController = controller((app) => {
  const router = app.getRouter();
  const ctrl = new HealthController({
    inject: {
      getHealthStatus: app.isHealthy.bind(app),
      responsesBuilder: app.get("responsesBuilder"),
      config: app.get("config"),
      statuses: app.get("statuses")
    }
  });
  return router.get("/", ctrl.showHealth());
});
export {
  HealthController,
  healthController
};
//# sourceMappingURL=health.js.map