import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { middlewareCreator } from "../../utils";
import { AppError } from "../../services";
class ErrorHandler {
  constructor({
    inject: { logger, responsesBuilder, statuses, HTTPError },
    ...options
  }) {
    __publicField(this, "_logger");
    __publicField(this, "_responsesBuilder");
    __publicField(this, "_statuses");
    __publicField(this, "_HTTPError");
    __publicField(this, "_options");
    this._logger = logger;
    this._responsesBuilder = responsesBuilder;
    this._statuses = statuses;
    this._HTTPError = HTTPError;
    this._options = deepAssignWithOverwrite(
      {
        showErrors: false,
        response: {
          message: "Unexpected error",
          status: this._statuses("internal server error")
        }
      },
      options
    );
  }
  getMiddleware() {
    return (err, _, res, next) => {
      if (!err) {
        next();
        return;
      }
      const { response, showErrors } = this._options;
      let { status } = response;
      let data = {
        error: true,
        message: response.message
      };
      const knownError = err instanceof AppError;
      if (showErrors || knownError) {
        data.message = err.message;
        if (knownError) {
          data = {
            ...data,
            ...err.getResponse()
          };
          status = err.status || this._statuses("bad request");
        }
        if (showErrors && err instanceof Error && err.stack) {
          const stack = err.stack.split("\n").map((line) => line.trim());
          data.stack = stack;
          stack.splice(0, 1);
          this._logger.error(`ERROR: ${err.message}`);
          this._logger.info(stack);
        }
      }
      this._responsesBuilder.json({
        res,
        data,
        status
      });
    };
  }
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
}
const errorHandlerMiddleware = middlewareCreator(
  (options = {}) => (app) => {
    const showErrors = app.getConfig("debug.showErrors") === true;
    return new ErrorHandler({
      inject: {
        logger: app.get("logger"),
        responsesBuilder: app.get("responsesBuilder"),
        statuses: app.get("statuses"),
        HTTPError: app.get("HTTPError")
      },
      showErrors,
      ...options
    }).getMiddleware();
  }
);
export {
  ErrorHandler,
  errorHandlerMiddleware
};
//# sourceMappingURL=errorHandler.js.map