import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { createRouteExpression, middlewareCreator, removeSlashes } from "../../utils";
class FastHTML {
  constructor({ inject, ...options }) {
    __publicField(this, "_sendFile");
    __publicField(this, "_events");
    __publicField(this, "_getHTMLGenerator");
    __publicField(this, "_options");
    __publicField(this, "_fileReady", false);
    __publicField(this, "_routeExpressions", []);
    this._sendFile = inject.sendFile;
    this._events = inject.events;
    this._getHTMLGenerator = inject.getHTMLGenerator || (() => void 0);
    this._options = {
      file: "index.html",
      ignoredRoutes: [/\.ico$/i],
      useAppRoutes: true,
      ...options
    };
    if (!this._options.ignoredRoutes.length && !this._options.useAppRoutes) {
      throw new Error("You must provide either `ignoredRoutes` or `useAppRoutes`");
    }
    if (this._options.useAppRoutes) {
      this._setupEvents();
    }
  }
  getMiddleware() {
    return async (req, res, next) => {
      if (this._shouldIgnoreRoute(req.originalUrl)) {
        next();
        return;
      }
      if (this._fileReady) {
        this._sendResponse(res, next);
        return;
      }
      const htmlGenerator = this._getHTMLGenerator();
      if (!htmlGenerator) {
        this._fileReady = true;
        this._sendResponse(res, next);
        return;
      }
      try {
        await htmlGenerator.whenReady();
        this._options.file = htmlGenerator.options.file;
        this._fileReady = true;
        this._sendResponse(res, next);
      } catch (error) {
        next(error);
      }
    };
  }
  get options() {
    return { ...this._options };
  }
  _sendResponse(res, next) {
    res.setHeader("Content-Type", "text/html");
    this._sendFile({
      res,
      next,
      filepath: this._options.file
    });
  }
  _setupEvents() {
    this._events.once("afterStart", ({ app }) => {
      const [routeExpressions] = app.routes.reduce(
        (acc, route) => {
          const [expressions, processed] = acc;
          const clean = removeSlashes(route).trim();
          if (!clean || processed.includes(clean))
            return acc;
          expressions.push(createRouteExpression(clean));
          processed.push(clean);
          return acc;
        },
        [[], []]
      );
      this._routeExpressions.push(...routeExpressions);
    });
  }
  _shouldIgnoreRoute(route) {
    return this._options.ignoredRoutes.some((expression) => route.match(expression)) || this._routeExpressions.some((expression) => route.match(expression));
  }
}
const fastHTMLMiddleware = middlewareCreator(
  (options = {}) => (app) => {
    const { htmlGeneratorServiceName = "htmlGenerator", ...rest } = options;
    return new FastHTML({
      inject: {
        events: app.get("events"),
        sendFile: app.get("sendFile"),
        getHTMLGenerator: () => app.try(htmlGeneratorServiceName)
      },
      ...rest
    }).getMiddleware();
  }
);
export {
  FastHTML,
  fastHTMLMiddleware
};
//# sourceMappingURL=fastHTML.js.map