import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { middlewareCreator } from "../../utils";
class ShowHTML {
  constructor({ inject, ...options }) {
    __publicField(this, "_sendFile");
    __publicField(this, "_getHTMLGenerator");
    __publicField(this, "_options");
    __publicField(this, "_fileReady", false);
    this._sendFile = inject.sendFile;
    this._getHTMLGenerator = inject.getHTMLGenerator || (() => void 0);
    this._options = {
      file: "index.html",
      ...options
    };
  }
  getMiddleware() {
    return async (_, res, next) => {
      if (this._fileReady) {
        this._sendResponse(res, next);
        return;
      }
      const htmlGenerator = this._getHTMLGenerator();
      if (!htmlGenerator) {
        this._fileReady = true;
        this._sendResponse(res, next);
        return;
      }
      try {
        await htmlGenerator.whenReady();
        this._options.file = htmlGenerator.options.file;
        this._fileReady = true;
        this._sendResponse(res, next);
      } catch (error) {
        next(error);
      }
    };
  }
  get options() {
    return { ...this._options };
  }
  _sendResponse(res, next) {
    res.setHeader("Content-Type", "text/html");
    this._sendFile({
      res,
      next,
      filepath: this._options.file
    });
  }
}
const showHTMLMiddleware = middlewareCreator(
  (options = {}) => (app) => {
    const { htmlGeneratorServiceName = "htmlGenerator", ...rest } = options;
    return new ShowHTML({
      inject: {
        sendFile: app.get("sendFile"),
        getHTMLGenerator: () => app.try(htmlGeneratorServiceName)
      },
      ...rest
    }).getMiddleware();
  }
);
export {
  ShowHTML,
  showHTMLMiddleware
};
//# sourceMappingURL=showHTML.js.map