import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { middlewareCreator } from "../../utils";
class VersionValidator {
  constructor({ inject, version, ...options }) {
    __publicField(this, "_HTTPError");
    __publicField(this, "_responsesBuilder");
    __publicField(this, "_statuses");
    __publicField(this, "_options");
    this._HTTPError = inject.HTTPError;
    this._responsesBuilder = inject.responsesBuilder;
    this._statuses = inject.statuses;
    this._options = deepAssignWithOverwrite(
      {
        error: "The application version doesn't match",
        latest: {
          allow: true,
          name: "latest"
        },
        popup: {
          variable: "popup",
          title: "Conflict",
          message: "vesion:conflict"
        },
        version
      },
      options
    );
    if (!this._options.version) {
      throw new Error("You need to supply a version");
    }
  }
  getMiddleware() {
    return (req, res, next) => {
      const { version } = req.params;
      if (!version) {
        next();
        return;
      }
      if (version === this._options.version || this._isTheAllowedLatest(version)) {
        next();
        return;
      }
      const status = this._statuses("conflict");
      if (this._isPopup(req)) {
        const { title, message } = this._options.popup;
        this._responsesBuilder.htmlPostMessage({
          res,
          title,
          message,
          status
        });
        return;
      }
      next(
        new this._HTTPError(this._options.error, status, {
          response: {
            validation: true
          }
        })
      );
    };
  }
  get options() {
    return deepAssignWithOverwrite({}, this._options);
  }
  _isPopup(req) {
    const popup = req.query[this._options.popup.variable];
    return !!(popup && String(popup).toLowerCase() === "true");
  }
  _isTheAllowedLatest(version) {
    const { allow, name } = this._options.latest;
    return allow && version === name;
  }
}
const versionValidatorMiddleware = middlewareCreator(
  (options = {}) => (app, route) => {
    const version = app.getConfig("version");
    const middleware = new VersionValidator({
      inject: {
        HTTPError: app.get("HTTPError"),
        responsesBuilder: app.get("responsesBuilder"),
        statuses: app.get("statuses")
      },
      version,
      ...options
    }).getMiddleware();
    if (route) {
      const router = app.getRouter();
      return router.all("/:version/*", middleware);
    }
    return middleware;
  }
);
export {
  VersionValidator,
  versionValidatorMiddleware
};
//# sourceMappingURL=versionValidator.js.map