import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { provider, statuses as statusesFn } from "../../utils";
class AppError extends Error {
  constructor(message, context = {}, statuses = statusesFn) {
    super(message);
    __publicField(this, "_date");
    __publicField(this, "_context");
    __publicField(this, "_statuses");
    this.name = this.constructor.name;
    this._date = new Date();
    this._statuses = statuses;
    this._context = this._parseContext(context);
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, this.constructor);
    }
  }
  getResponse() {
    return this.response;
  }
  get response() {
    return this._context.response || {};
  }
  get status() {
    return this._context.status;
  }
  get context() {
    return this._context;
  }
  get date() {
    return this._date;
  }
  _parseContext(context) {
    const result = { ...context };
    if (result.status && typeof result.status === "string") {
      result.status = this._statuses.code[result.status.toLowerCase()] || result.status;
    }
    return result;
  }
}
const createAppError = (...args) => new AppError(...args);
const appErrorProvider = provider((app) => {
  app.set("AppError", () => AppError);
  app.set("appError", () => createAppError);
});
export {
  AppError,
  appErrorProvider,
  createAppError
};
//# sourceMappingURL=appError.js.map