import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import {
  APIClient as APIClientBase
} from "@homer0/api-utils";
import { deepAssignWithOverwrite } from "@homer0/deep-assign";
import { providerCreator } from "../../utils";
class APIClient extends APIClientBase {
  constructor({ inject: { http, HTTPError }, ...rest }) {
    const { endpoints, gateway, url } = rest;
    const useEndpoints = endpoints || gateway;
    const apiConfig = {
      url,
      endpoints: useEndpoints
    };
    super({
      ...apiConfig,
      fetchClient: http.fetch
    });
    __publicField(this, "_http");
    __publicField(this, "_HTTPError");
    __publicField(this, "_apiConfig");
    this._http = http;
    this._HTTPError = HTTPError;
    this._apiConfig = deepAssignWithOverwrite({}, apiConfig);
  }
  get apiConfig() {
    return deepAssignWithOverwrite({}, this._apiConfig);
  }
  _getErrorMessageFromResponse(response) {
    const res = response;
    if (res.error)
      return res.error;
    if (res.data) {
      if (res.data.message)
        return res.data.message;
      if (res.data.error)
        return res.data.error;
    }
    return "Unexpected error";
  }
  formatError(response, status) {
    return new this._HTTPError(this._getErrorMessageFromResponse(response), status);
  }
}
const apiClientProvider = providerCreator(
  (options = {}) => (app) => {
    const defaultName = "apiClient";
    const { serviceName = defaultName, clientClass: ClientClass = APIClient } = options;
    let { configSetting } = options;
    if (!configSetting) {
      configSetting = serviceName === defaultName ? "api" : serviceName;
    }
    app.set(
      serviceName,
      () => new ClientClass({
        inject: {
          http: app.get("http"),
          HTTPError: app.get("HTTPError")
        },
        ...app.getConfig(configSetting)
      })
    );
  }
);
export {
  APIClient,
  apiClientProvider
};
//# sourceMappingURL=apiClient.js.map