import {
  __publicField
} from "../../chunk-T2T6Q22Z.js";
import { provider } from "../../utils";
const DEFAULT_CLOSE_DELAY_FOR_POST_MESSAGE = 700;
class ResponsesBuilder {
  constructor({ inject: { config, statuses } }) {
    __publicField(this, "_config");
    __publicField(this, "_statuses");
    this._config = config;
    this._statuses = statuses;
  }
  htmlPostMessage(options) {
    const {
      res,
      title,
      message,
      status,
      target = "window.opener",
      close = true,
      closeDelay = DEFAULT_CLOSE_DELAY_FOR_POST_MESSAGE
    } = options;
    const prefix = this._config.get("postMessagesPrefix") ?? "";
    const closeCode = close ? `setTimeout(function() { window.close(); }, ${closeDelay});` : "";
    const html = this._htmlTemplate(
      title,
      `
      (function() {
        if (${target}) {
          ${target}.postMessage('${prefix}${message}', '*');
          ${closeCode}
        }
      })();
      `
    );
    const useStatus = typeof status === "undefined" ? this._statuses("ok") : this._normalizeStatus(status);
    res.setHeader("Content-Type", "text/html");
    res.setHeader("Cache-Control", "no-cache, max-age=0, must-revalidate, no-store");
    res.status(useStatus);
    res.write(html);
    res.end();
  }
  json(options) {
    const { res, data, status, metadata = {} } = options;
    const useStatus = typeof status === "undefined" ? this._statuses("ok") : this._normalizeStatus(status);
    res.status(useStatus);
    res.json({
      metadata: {
        version: this._config.get("version"),
        status: useStatus,
        ...metadata
      },
      data
    });
    res.end();
  }
  _htmlTemplate(title, code) {
    return `
      <!DOCTYPE html>
      <html>
        <head>
          <meta charset="utf-8" />
          <meta http-equiv="x-ua-compatible" content="ie=edge" />
          <meta name="viewport" content="width=device-width, initial-scale=1" />
          <title>${title}</title>
        </head>
        <body>
          <script type="text/javascript">
          ${code}
        <\/script>
      </body>
    </html>
    `;
  }
  _normalizeStatus(status) {
    let useStatus;
    try {
      if (typeof status === "string") {
        useStatus = this._statuses(status);
      } else {
        this._statuses(status);
        useStatus = status;
      }
    } catch (_) {
      useStatus = this._statuses("ok");
    }
    return useStatus;
  }
}
const responsesBuilderProvider = provider((app) => {
  app.set(
    "responsesBuilder",
    () => new ResponsesBuilder({
      inject: {
        config: app.getConfig(),
        statuses: app.get("statuses")
      }
    })
  );
});
export {
  ResponsesBuilder,
  responsesBuilderProvider
};
//# sourceMappingURL=responsesBuilder.js.map