"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var fastHTML_exports = {};
__export(fastHTML_exports, {
  FastHTML: () => FastHTML,
  fastHTMLMiddleware: () => fastHTMLMiddleware
});
module.exports = __toCommonJS(fastHTML_exports);
var import_utils = require("../../utils");
class FastHTML {
  constructor({ inject, ...options }) {
    __publicField(this, "_sendFile");
    __publicField(this, "_events");
    __publicField(this, "_getHTMLGenerator");
    __publicField(this, "_options");
    __publicField(this, "_fileReady", false);
    __publicField(this, "_routeExpressions", []);
    this._sendFile = inject.sendFile;
    this._events = inject.events;
    this._getHTMLGenerator = inject.getHTMLGenerator || (() => void 0);
    this._options = {
      file: "index.html",
      ignoredRoutes: [/\.ico$/i],
      useAppRoutes: true,
      ...options
    };
    if (!this._options.ignoredRoutes.length && !this._options.useAppRoutes) {
      throw new Error("You must provide either `ignoredRoutes` or `useAppRoutes`");
    }
    if (this._options.useAppRoutes) {
      this._setupEvents();
    }
  }
  getMiddleware() {
    return async (req, res, next) => {
      if (this._shouldIgnoreRoute(req.originalUrl)) {
        next();
        return;
      }
      if (this._fileReady) {
        this._sendResponse(res, next);
        return;
      }
      const htmlGenerator = this._getHTMLGenerator();
      if (!htmlGenerator) {
        this._fileReady = true;
        this._sendResponse(res, next);
        return;
      }
      try {
        await htmlGenerator.whenReady();
        this._options.file = htmlGenerator.options.file;
        this._fileReady = true;
        this._sendResponse(res, next);
      } catch (error) {
        next(error);
      }
    };
  }
  get options() {
    return { ...this._options };
  }
  _sendResponse(res, next) {
    res.setHeader("Content-Type", "text/html");
    this._sendFile({
      res,
      next,
      filepath: this._options.file
    });
  }
  _setupEvents() {
    this._events.once("afterStart", ({ app }) => {
      const [routeExpressions] = app.routes.reduce(
        (acc, route) => {
          const [expressions, processed] = acc;
          const clean = (0, import_utils.removeSlashes)(route).trim();
          if (!clean || processed.includes(clean))
            return acc;
          expressions.push((0, import_utils.createRouteExpression)(clean));
          processed.push(clean);
          return acc;
        },
        [[], []]
      );
      this._routeExpressions.push(...routeExpressions);
    });
  }
  _shouldIgnoreRoute(route) {
    return this._options.ignoredRoutes.some((expression) => route.match(expression)) || this._routeExpressions.some((expression) => route.match(expression));
  }
}
const fastHTMLMiddleware = (0, import_utils.middlewareCreator)(
  (options = {}) => (app) => {
    const { htmlGeneratorServiceName = "htmlGenerator", ...rest } = options;
    return new FastHTML({
      inject: {
        events: app.get("events"),
        sendFile: app.get("sendFile"),
        getHTMLGenerator: () => app.try(htmlGeneratorServiceName)
      },
      ...rest
    }).getMiddleware();
  }
);
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  FastHTML,
  fastHTMLMiddleware
});
//# sourceMappingURL=fastHTML.js.map