"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
var responsesBuilder_exports = {};
__export(responsesBuilder_exports, {
  ResponsesBuilder: () => ResponsesBuilder,
  responsesBuilderProvider: () => responsesBuilderProvider
});
module.exports = __toCommonJS(responsesBuilder_exports);
var import_utils = require("../../utils");
const DEFAULT_CLOSE_DELAY_FOR_POST_MESSAGE = 700;
class ResponsesBuilder {
  constructor({ inject: { config, statuses } }) {
    __publicField(this, "_config");
    __publicField(this, "_statuses");
    this._config = config;
    this._statuses = statuses;
  }
  htmlPostMessage(options) {
    const {
      res,
      title,
      message,
      status,
      target = "window.opener",
      close = true,
      closeDelay = DEFAULT_CLOSE_DELAY_FOR_POST_MESSAGE
    } = options;
    const prefix = this._config.get("postMessagesPrefix") ?? "";
    const closeCode = close ? `setTimeout(function() { window.close(); }, ${closeDelay});` : "";
    const html = this._htmlTemplate(
      title,
      `
      (function() {
        if (${target}) {
          ${target}.postMessage('${prefix}${message}', '*');
          ${closeCode}
        }
      })();
      `
    );
    const useStatus = typeof status === "undefined" ? this._statuses("ok") : this._normalizeStatus(status);
    res.setHeader("Content-Type", "text/html");
    res.setHeader("Cache-Control", "no-cache, max-age=0, must-revalidate, no-store");
    res.status(useStatus);
    res.write(html);
    res.end();
  }
  json(options) {
    const { res, data, status, metadata = {} } = options;
    const useStatus = typeof status === "undefined" ? this._statuses("ok") : this._normalizeStatus(status);
    res.status(useStatus);
    res.json({
      metadata: {
        version: this._config.get("version"),
        status: useStatus,
        ...metadata
      },
      data
    });
    res.end();
  }
  _htmlTemplate(title, code) {
    return `
      <!DOCTYPE html>
      <html>
        <head>
          <meta charset="utf-8" />
          <meta http-equiv="x-ua-compatible" content="ie=edge" />
          <meta name="viewport" content="width=device-width, initial-scale=1" />
          <title>${title}</title>
        </head>
        <body>
          <script type="text/javascript">
          ${code}
        <\/script>
      </body>
    </html>
    `;
  }
  _normalizeStatus(status) {
    let useStatus;
    try {
      if (typeof status === "string") {
        useStatus = this._statuses(status);
      } else {
        this._statuses(status);
        useStatus = status;
      }
    } catch (_) {
      useStatus = this._statuses("ok");
    }
    return useStatus;
  }
}
const responsesBuilderProvider = (0, import_utils.provider)((app) => {
  app.set(
    "responsesBuilder",
    () => new ResponsesBuilder({
      inject: {
        config: app.getConfig(),
        statuses: app.get("statuses")
      }
    })
  );
});
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  ResponsesBuilder,
  responsesBuilderProvider
});
//# sourceMappingURL=responsesBuilder.js.map